(function() {
    var elementSourceMap = new WeakMap();

    var originalCreateElement = document.createElement;
    document.createElement = function(tagName, options) {
        var ret = XWebViewApi.invokeCommand('interruptElementCreate', tagName);
        if (ret === true) {
            return null;
        }

        var element = originalCreateElement.call(document, tagName, options);
        if (XWebViewApi.invokeCommand('needRecordElementStack', 'createElement') === true) {
            elementSourceMap.set(element, new Error('Element creation trace').stack);
        }

        XWebViewApi.invokeCommand('notifyElementCreated', element);
        return element;
    };

    var originalAppendChild = Node.prototype.appendChild;
    Node.prototype.appendChild = function(child) {
        if (child.nodeType === Node.ELEMENT_NODE && !elementSourceMap.has(child)) {
            if (XWebViewApi.invokeCommand('needRecordElementStack', 'appendChild') === true) {
                elementSourceMap.set(child, new Error('Element creation trace').stack);
            }
            XWebViewApi.invokeCommand('notifyElementCreated', child);
        }
        return originalAppendChild.call(this, child);
    };

    var originalInsertBefore = Node.prototype.insertBefore;
    Node.prototype.insertBefore = function(newNode, referenceNode) {
        if (newNode.nodeType === Node.ELEMENT_NODE && !elementSourceMap.has(newNode)) {
            if (XWebViewApi.invokeCommand('needRecordElementStack', 'insertBefore') === true) {
                elementSourceMap.set(newNode, new Error('Element creation trace').stack);
            }
            XWebViewApi.invokeCommand('notifyElementCreated', newNode);
        }
        return originalInsertBefore.call(this, newNode, referenceNode);
    };

    var originalReplaceChild = Node.prototype.replaceChild;
    Node.prototype.replaceChild = function(newChild, oldChild) {
        if (newChild.nodeType === Node.ELEMENT_NODE && !elementSourceMap.has(newChild)) {
            if (XWebViewApi.invokeCommand('needRecordElementStack', 'replaceChild') === true) {
                elementSourceMap.set(newChild, new Error('Element creation trace').stack);
            }
            XWebViewApi.invokeCommand('notifyElementCreated', newChild);
        }
        elementSourceMap.delete(oldChild);
        return originalReplaceChild.call(this, newChild, oldChild);
    };

    var originalRemoveChild = Node.prototype.removeChild;
    Node.prototype.removeChild = function(child) {
        var ret = XWebViewApi.invokeCommand('interruptElementRemove', child);
        if (ret === true) {
            return;
        }

        elementSourceMap.delete(child);
        return originalRemoveChild.call(this, child);
    };

    XWebViewApi.registerCommand('getElementStackTrace', function(element) {
        return elementSourceMap.get(element);
    });
})();