(function() {
    if (window.__PIP_GC == null) {
        window.__PIP_GC = [];

        var pictureInPictureElement = null;
        var pictureInPictureWindow = {
            width:0,
            height:0,
            resize: [],
            fire: function() {
                var event = new Event("resize");
                event.target = pictureInPictureWindow;
                event.width = pictureInPictureWindow.width;
                event.height = pictureInPictureWindow.height;

                for (var i = 0; i < pictureInPictureWindow.resize.length; ++i) {
                    pictureInPictureWindow.resize[i](event);
                }
            }
        };
        Object.defineProperty(pictureInPictureWindow, "addEventListener", {
            get: function() {
                return function(arg1, arg2, arg3, arg4) {
                    console.log("pictureInPictureWindow.addEventListener:" + arg1);
                    pictureInPictureWindow.resize.push(arg2);
                };
            }
        });
        Object.defineProperty(pictureInPictureWindow, "removeEventListener", {
            get: function() {
                return function(arg1, arg2, arg3, arg4) {
                    console.log("pictureInPictureWindow.removeEventListener:" + arg1);
                    const index = pictureInPictureWindow.resize.indexOf(arg2);
                    if (index > -1) {
                        pictureInPictureWindow.resize.splice(index, 1);
                    }
                };
            }
        });

        function getVideoSrc(ele) {
            var videoEle = (ele instanceof HTMLVideoElement) ? ele : (ele.target ? ele.target : ele);
            var videoSrc = videoEle.src;
            if (!videoSrc || videoSrc == "" || !videoSrc.startsWith("http")) {
                var c = videoEle.querySelector("source");
                if (c) {
                    videoSrc = c.src;
                }
            }
            return !videoSrc ? "" : videoSrc;
        }

        function pictureInPictureEnabled() {
            return XWebViewApi.call("pictureInPictureEnabled") == 'true';
        }

        function exitPictureInPicture() {
            console.log('exitPictureInPicture');

            var video = pictureInPictureElement;
            pictureInPictureElement = null;
            if (video != null) {
                var event = new Event("leavepictureinpicture");
                video.video.dispatchEvent(event);

                XWebViewApi.call("exitPictureInPicture", video);
                window.__PIP_GC[video.id] = null;
            }
        }

        function disablePictureInPicture(ele) {
            return !getVideoSrc(ele).startsWith("http");
        }

        function requestPictureInPicture(video) {
            if (video.disablePictureInPicture) {
                console.log('requestPictureInPicture ignore with video disablePictureInPicture!');
                return null
            }
            console.log('requestPictureInPicture:' + video);

            var videoEle = (video instanceof HTMLVideoElement) ? video : (video.target ? video.target : video);
            var videoSrc = getVideoSrc(videoEle);

            pictureInPictureElement = {};
            pictureInPictureElement.id = 'pictureInPictureElement@' + new Date().getTime();
            pictureInPictureElement.video = videoEle;
            pictureInPictureElement.url = videoSrc;
            pictureInPictureElement.progress = videoEle.currentTime;
            pictureInPictureElement.duration = videoEle.duration;
            pictureInPictureElement.width = videoEle.videoWidth;
            pictureInPictureElement.height = videoEle.videoHeight;
            pictureInPictureElement.title = videoEle.ownerDocument.title;
            pictureInPictureElement.page = location.href;
            pictureInPictureElement.cookie = videoEle.ownerDocument.cookie;
            pictureInPictureElement.userAgent = window.navigator.userAgent;
            pictureInPictureElement.callback = 'window.__PIP_GC["' + pictureInPictureElement.id + '"]';

            window.__PIP_GC[pictureInPictureElement.id] = function(params) {
                console.log('requestPictureInPicture callback invoke:' + params);

                var args = JSON.parse(params);
                if (args.action == 'play') {
                    try { pictureInPictureElement.video.play() } catch (e) {console.log(e);}
                } else if (args.action == 'pause') {
                    try { pictureInPictureElement.video.pause(); } catch (e) {console.log(e);}
                } else if (args.action == 'seek') {
                    if (pictureInPictureElement.video.fastSeek) {
                        try { pictureInPictureElement.video.fastSeek(args.time); } catch (e) {console.log(e);}
                    } else {
                        pictureInPictureElement.video.currentTime = args.time
                    }
                } else if (args.action == 'exitPip') {
                    exitPictureInPicture();
                }
            };

            XWebViewApi.call("requestPictureInPicture", pictureInPictureElement);

            var event = new Event("enterpictureinpicture");
            event.pictureInPictureWindow = pictureInPictureWindow;
            pictureInPictureElement.video.dispatchEvent(event);

            pictureInPictureWindow.width = pictureInPictureElement.width;
            pictureInPictureWindow.height = pictureInPictureElement.height;
            pictureInPictureWindow.fire();
            return pictureInPictureWindow;
        }

        function injectDocument(doc) {
            if (!doc) {
                return;
            }

            if (!!doc && !doc.pictureInPictureElement) {
                Object.defineProperty(doc, "pictureInPictureElement", {
                    get: function() {
                        return pictureInPictureElement;
                    }
                });
                Object.defineProperty(doc, "pictureInPictureEnabled", {
                    get: function() {
                        return pictureInPictureEnabled();
                    }
                });
                Object.defineProperty(doc, "exitPictureInPicture", {
                    get: function() {
                        return exitPictureInPicture;
                    }
                });

                var iframes = doc.getElementsByTagName('iframe');
                for (var i = 0; iframes && i < iframes.length; ++i) {
                    if (iframes[i].contentDocument) {
                        injectDocument(iframes[i].contentDocument);
                    }
                }

                new MutationObserver(function(mutationsList) {
                    for (let mutation of mutationsList) {
                        if (mutation.type === 'childList') {
                            if (mutation.addedNodes != null) {
                                for (let node of mutation.addedNodes) {
                                    if (node.tagName == 'IFRAME') {
                                        injectDocument(node.contentDocument);
                                    }
                                }
                            }
                        }
                    }
                }).observe(doc, {subtree: true, childList: true});
            }
        };

        function injectElement() {
            Object.defineProperty(HTMLVideoElement.prototype, 'requestPictureInPicture', {
                get: function() { // this is video object
                    var that = this;
                    return function() {
                        return new Promise(function(r, j) {
                            r(requestPictureInPicture(that));
                        });
                    };
                }
            });
            Object.defineProperty(HTMLVideoElement.prototype, "disablePictureInPicture", {
                get: function() {
                    return disablePictureInPicture(this);
                },
                set: function(value) {

                }
            });
        }

        injectDocument(document);
        injectElement();
    }
})();