(function() {
    if (window.speechSynthesis == null) {
        class SpeechSynthesisVoice {
            constructor(id, def, name, lang, localService, voiceURI) {
                this._id = id;
                this._name = name;
                this._default = def; //false;
                this._lang = lang; //'en-US';
                this._localService = localService; //true;
                this._voiceURI = voiceURI; //'urn:moz-tts:osx:com.apple.speech.synthesis.voice.daniel';
            }

            get id() {
                return this._id;
            }

            get 'default'() {
                return this._default;
            }

            get name() {
                return this._name;
            }

            get lang() {
                return this._lang;
            }

            get localService() {
                return this._localService;
            }

            get voiceURI() {
                return this._voiceURI;
            }
        }

        class SpeechSynthesisUtterance extends EventTarget {
            constructor(val) {
                super();
                this._lang = '';
                this._pitch = 0;
                this._rate = 0;
                this._volume = 0;
                this._text = val;
                this._utterance = 'utterance:' + new Date().getTime();
            }

            get utterance() {
                return this._utterance;
            }

            set text(val) {
                this._text = val;
            }

            get text() {
                return this._text;
            }

            set lang(val) {
                this._lang = val;
            }

            get lang() {
                return this._lang;
            }

            set pitch(val) {
                this._pitch = val;
            }

            get pitch() {
                return this._pitch;
            }

            set rate(val) {
                this._rate = val;
            }

            get rate() {
                return this._rate;
            }

            set voice(val) {
                this._voice = val;
            }

            get voice() {
                return this._voice;
            }

            set volume(val) {
                this._volume = val;
            }

            get volume() {
                return this._volume;
            }

            // e: boundary|end|error|mark|pause|resume|start
            emitEvent(e) {
                var event = new CustomEvent(e);
                event.charIndex = 0;
                event.elapsedTime = 0;
                event.name = '';
                event.utterance = this.utterance;
                this.dispatchEvent(event);
            }
        }

        class SpeechSynthesis extends EventTarget {
            constructor() {
                super();
                this._paused = false;
                this._pending = false;
                this._speaking = false;
                this._utterances = {};

                var event = {};
                event.name = 'voiceschanged';
                event.callback = 'window.speechSynthesis.onvoiceschanged';
                this.nativeCall('attachEvent', event);

                var event = {};
                event.name = 'utteranceprogress';
                event.callback = 'window.speechSynthesis.onutteranceprogress';
                this.nativeCall('attachEvent', event);
            }

            get speaking() {
                return this._speaking;
            }

            get paused() {
                return this._paused;
            }

            get pending() {
                return this._pending;
            }

            cancel() {
                this.nativeCall('cancel', {});
            }

            getVoices() {
                var list = [];
                var ret = XWebViewApi.parse(this.nativeCall('getVoices', {}), []);
                for (var i = 0; i < ret.length; ++i) {
                    var item = ret[i];
                    var voice = new SpeechSynthesisVoice(item.id, item["default"], item.name, item.lang, true, item.voiceURI);
                    list.push(voice);
                }
                return list;
            }

            pause() {
                this.nativeCall('pause', {});
            }

            resume() {
                this.nativeCall('resume', {});
            }

            speak(u) {
                this._utterances[u.utterance] = u;

                var item = {};
                item.utteranceId = u.utterance;
                item.text = u.text;
                item.rate = u.rate;
                item.lang = u.lang;
                item.pitch = u.pitch;
                item.volume = u.volume;
                this.nativeCall('speak', item);
            }

            onvoiceschanged(voice) {
                var event = new CustomEvent('voiceschanged');
                this.dispatchEvent(event);
            }

            onutteranceprogress(e) {
                var _e = XWebViewApi.parse(e, {});
                var name = _e.event;
                var utteranceId = _e.utteranceId;
                var utterance = this._utterances[utteranceId];

                if ("start" == name) {
                    this._speaking = true;
                } else if ("error" == name) {
                    this._speaking = false;
                    this._utterances[utteranceId] = null;
                } else if ("done" == name) {
                    this._speaking = false;
                    this._utterances[utteranceId] = null;
                }

                if (utterance != null) {
                    utterance.emitEvent(name);
                }

                var event = new CustomEvent('utteranceprogress');
                event.name = name;
                event.utteranceId = utteranceId;
                this.dispatchEvent(event);
            }

            nativeCall(fun, params) {
                // fun, param, plugin, object
                return XWebViewApi.call(fun, params, null, 'tts');
            }
        }

        window.SpeechSynthesisVoice = SpeechSynthesisVoice;
        window.SpeechSynthesisUtterance = SpeechSynthesisUtterance;
        window.speechSynthesis = new SpeechSynthesis();
    }
})();