(function() {
    if (window.customElements.get('x-video') == null) {
        function getScrollParent(node) {
            var regex = /(auto|scroll)/;
            var parents = (_node, ps) => {
                if (_node.parentNode === null) { return ps; }
                return parents(_node.parentNode, ps.concat([_node]));
            };

            var style = (_node, prop) => getComputedStyle(_node, null).getPropertyValue(prop);
            var overflow = _node => style(_node, 'overflow') + style(_node, 'overflow-y') + style(_node, 'overflow-x');
            var scroll = _node => regex.test(overflow(_node));

            var scrollParent = (_node) => {
                if (!(_node instanceof HTMLElement || _node instanceof SVGElement)) {
                    return;
                }

                var ps = parents(_node.parentNode, []);
                for (var i = 0; i < ps.length; i += 1) {
                    if (scroll(ps[i])) {
                        return ps[i];
                    }
                }

                return document.scrollingElement || document.documentElement;
            };

            return scrollParent(node);
        };

        function invokeEleMethod(ele, method, params) {
            var video = {};
            video.tag = ele.tagName.toLowerCase();
            video.id = ele.id;
            video.method = method;
            video.params = params;
            XWebViewApi.call("invokeCustomElementMethod", JSON.stringify(video));
        }

        function notifyEleChanged(ele) {
            var rect = ele.getBoundingClientRect();
            var video = {};
            video.tag = ele.tagName.toLowerCase();
            video.id = ele.id;
            video.width = rect.width;
            video.height = rect.height;
            video.left = rect.left;
            video.top = rect.top;
            video.right = rect.right;
            video.bottom = rect.bottom;
            video.src = ele.src;
            video.referer = ele.referer;
            video.origin = ele.origin;
            video.playid = ele.playid;
            video.playlist = ele.playlist;
            video.page = ele.ownerDocument.location.href;
            video.title = ele.ownerDocument.title;
            video.cookie = ele.ownerDocument.cookie;
            video.userAgent = window.navigator.userAgent;
            video.hidden = document.hidden;
            video.autoplay = true || ele.hasAttribute('autoplay');
            video.debugColor = ele.getAttribute('debug-color');
            video.debugColor = video.debugColor == null ? '#00000000' : video.debugColor;
            video.devicePixelRatio = window.devicePixelRatio;
            video.navigationType = performance.navigation.type;
            XWebViewApi.call("notifyCustomElementChanged", JSON.stringify(video));

            var p = getScrollParent(ele);
            if (p != null && p.hasEvent == null) {
                p.hasEvent = true;
                p.addEventListener('scroll', function() {
                    notifyEleChanged(ele);
                }, false);
            }
        };

        function notifyAllTagChanged(tag) {
            var v = document.getElementsByTagName(tag);
            for (var i = 0; i < v.length; ++i) {
                notifyEleChanged(v[i]);
            }
        };

        function notifyAllChanged() {
            notifyAllTagChanged('x-video');
            //notifyAllTagChanged('video');
        }

        class XVideoElement extends HTMLElement {
            constructor() {
                super();

                var that = this;
                new PositionObserver().observe(this, (dom, pos) => {
                    notifyEleChanged(that);
                });

                console.log('XVideoElement constructor, src=', this.src);
                if (this.src != null && this.src.length > 0) {
                    this._tryGetUrl(this.src);
                }
            }

            get id() {
                var ret = this.getAttribute('id');
                if (ret == null) {
                    if (XWebViewApi.customElements == null) {
                        XWebViewApi.customElements = 1;
                    } else {
                        XWebViewApi.customElements += 1;
                    }
                    this.setAttribute('id', ret = 'x-video-' + XWebViewApi.customElements);
                }
                return ret;
            }

            set id(val) {
                console.error('Not allow set id !!!');
            }

            get src() {
                return this.getAttribute('src');
            }

            set src(val) {
                this.setAttribute('src', val);
            }

            get referer() {
                return this.getAttribute('referer');
            }

            set referer(val) {
                this.setAttribute('referer', val);
            }

            get origin() {
                return this.getAttribute('origin');
            }

            set origin(val) {
                this.setAttribute('origin', val);
            }

            set style(val) {
                //console.log('set style:' + val);
                this.setAttribute('style', val);
            }

            setAttribute(k, v) {
                console.log('XVideoElement setAttribute, k=', k, 'v=', v);
                if ('src' === k && v != null && v.length > 0 && this.src != null && this.src !== v) {
                    this._tryGetUrl(v);
                }
                super.setAttribute(k, v);
                notifyEleChanged(this);
            }

            play() {
                notifyEleChanged(this);
            }

            requestFullscreen() {
                invokeEleMethod(this, 'requestFullscreen', '');
            }

            exitFullscreen() {
                invokeEleMethod(this, 'exitFullscreen', '');
            }

            _tryGetUrl(url) {
                try {
                   var xhr = new XMLHttpRequest();
                   xhr.open('get', url, true);
                   xhr.send();
                   xhr.onprogress = function() {
                       xhr.abort();
                   };
                } catch (e) {
                    console.error(e);
                }
            }
        };

        window.customElements.define('x-video', XVideoElement);
        window.addEventListener('scroll', function() {
            notifyAllChanged();
        }, true);
        document.addEventListener("visibilitychange", function() {
            console.log('visibilitychange');
            notifyAllChanged();
        }, true);
        XWebViewApi.registerCommand('readystatechange', function() {
            console.log('readystatechange');
            notifyAllChanged();
        });
        XWebViewApi.registerCommand('documentload', function() {
            console.log('documentload');
            notifyAllChanged();
        });
        XWebViewApi.registerCommand('elementchange', function(ele) {
            if (ele && ele.tagName && (ele.tagName.toLowerCase() == 'x-video'/* || ele.tagName.toLowerCase() == 'video'*/)) {
                notifyEleChanged(ele);
            }
        });
        notifyAllChanged();
    }
})();